@@ -32,7 +32,8 @@ enum {
 	PROP_HTML_MODE,
 	PROP_INLINE_SPELLING,
 	PROP_MAGIC_LINKS,
-	PROP_MAGIC_SMILEYS
+	PROP_MAGIC_SMILEYS,
+	PROP_CHANGED
 };
 
 enum {
@@ -216,6 +217,12 @@ editor_url_requested_cb (GtkhtmlEditor *editor,
 	g_signal_emit (editor, signals[URI_REQUESTED], 0, url, stream);
 }
 
+static void
+engine_undo_changed_cb (GtkhtmlEditor *editor, HTMLEngine *engine)
+{
+	g_object_notify (G_OBJECT (editor), "changed");
+}
+
 static void
 editor_command_popup_menu (GtkhtmlEditor *editor)
 {
@@ -413,10 +420,11 @@ editor_method_set_language (GtkHTML *html,
 	action = gtk_action_group_get_action (action_group, action_name);
 	g_free (action_name);
 
+	/* silently ignore when such language doesn't exist, it might mean that
+	   user doesn't have installed language for locale the editor is run in
+	*/
 	if (action != NULL)
 		gtk_action_activate (action);
-	else
-		g_warning ("%s: No such language", language);
 }
 
 static GtkHTMLEditorAPI editor_api = {
@@ -500,6 +508,10 @@ editor_constructor (GType type,
 		html, "url_requested",
 		G_CALLBACK (editor_url_requested_cb), editor);
 
+	g_signal_connect_swapped (
+		html->engine, "undo-changed",
+		G_CALLBACK (engine_undo_changed_cb), editor);
+
 	/* Connect property dialog widgets to actions. */
 
 	gtk_activatable_set_related_action (
@@ -644,6 +656,11 @@ editor_set_property (GObject *object,
 				GTKHTML_EDITOR (object),
 				g_value_get_boolean (value));
 			return;
+		case PROP_CHANGED:
+			gtkhtml_editor_set_changed (
+				GTKHTML_EDITOR (object),
+				g_value_get_boolean (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -697,6 +714,11 @@ editor_get_property (GObject *object,
 				value, gtkhtml_editor_get_magic_smileys (
 				GTKHTML_EDITOR (object)));
 			return;
+		case PROP_CHANGED:
+			g_value_set_boolean (
+				value, gtkhtml_editor_get_changed (
+				GTKHTML_EDITOR (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -726,6 +748,23 @@ editor_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+editor_key_press_event (GtkWidget *widget,
+                        GdkEventKey *event)
+{
+#ifdef HAVE_XFREE
+	GtkhtmlEditor *editor = GTKHTML_EDITOR (widget);
+
+	if (event->keyval == XF86XK_Spell) {
+		gtk_action_activate (ACTION (SPELL_CHECK));
+		return TRUE;
+	}
+#endif
+
+	/* Chain up to parent's key_press_event() method. */
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+}
+
 static void
 editor_cut_clipboard (GtkhtmlEditor *editor)
 {
@@ -754,6 +793,7 @@ static void
 editor_class_init (GtkhtmlEditorClass *class)
 {
 	GObjectClass *object_class;
+	GtkWidgetClass *widget_class;
 
 	parent_class = g_type_class_peek_parent (class);
 	g_type_class_add_private (class, sizeof (GtkhtmlEditorPrivate));
@@ -765,6 +805,9 @@ editor_class_init (GtkhtmlEditorClass *class)
 	object_class->dispose = editor_dispose;
 	object_class->finalize = editor_finalize;
 
+	widget_class = GTK_WIDGET_CLASS (class);
+	widget_class->key_press_event = editor_key_press_event;
+
 	class->cut_clipboard = editor_cut_clipboard;
 	class->copy_clipboard = editor_copy_clipboard;
 	class->paste_clipboard = editor_paste_clipboard;
@@ -847,6 +890,16 @@ editor_class_init (GtkhtmlEditorClass *class)
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_CHANGED,
+		g_param_spec_boolean (
+			"changed",
+			_("Changed property"),
+			_("Whether editor changed"),
+			FALSE,
+			G_PARAM_READWRITE));
+
 	signals[COMMAND_AFTER] = g_signal_new (
 		"command-after",
 		G_OBJECT_CLASS_TYPE (class),
@@ -911,7 +964,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 static void
 editor_init (GtkhtmlEditor *editor)
 {
-	editor->priv = GTKHTML_EDITOR_GET_PRIVATE (editor);
+	editor->priv = G_TYPE_INSTANCE_GET_PRIVATE (
+		editor, GTKHTML_TYPE_EDITOR, GtkhtmlEditorPrivate);
 	editor->vbox = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
 	gtk_widget_show (editor->vbox);
 
@@ -1108,6 +1162,8 @@ gtkhtml_editor_set_changed (GtkhtmlEditor *editor,
 	}
 
 	editor->priv->changed = changed;
+
+	g_object_notify (G_OBJECT (editor), "changed");
 }
 
 const gchar *
@@ -1726,7 +1782,7 @@ gtkhtml_editor_undo_end (GtkhtmlEditor *editor)
 
 	html = gtkhtml_editor_get_html (editor);
 
-	html_undo_level_end (html->engine->undo);
+	html_undo_level_end (html->engine->undo, html->engine);
 }
 
 gboolean
