@@ -57,8 +57,8 @@ get_locale_language (void)
 		if (strlen (lang) >= 2) {
 			if (lang[2] == '-' || lang[2] == '_') {
 				/* canonicalise the lang */
-				lang [0] = g_ascii_tolower (lang [0]);
-				lang [1] = g_ascii_tolower (lang [1]);
+				lang[0] = g_ascii_tolower (lang[0]);
+				lang[1] = g_ascii_tolower (lang[1]);
 
 				/* validate the country code */
 				if (strlen (lang + 3) > 2) {
@@ -66,8 +66,8 @@ get_locale_language (void)
 					lang[2] = '\0';
 				} else {
 					lang[2] = '-';
-					lang [3] = g_ascii_toupper (lang [3]);
-					lang [4] = g_ascii_toupper (lang [4]);
+					lang[3] = g_ascii_toupper (lang[3]);
+					lang[4] = g_ascii_toupper (lang[4]);
 				}
 			} else if (lang[2] != '\0') {
 				/* invalid language */
@@ -86,7 +86,7 @@ get_locale_language (void)
 }
 
 GtkHTMLClassProperties *
-gtk_html_class_properties_new (GtkWidget *widget)
+gtk_html_class_properties_new (void)
 {
 	GtkHTMLClassProperties *p = g_new0 (GtkHTMLClassProperties, 1);
 
@@ -102,21 +102,6 @@ gtk_html_class_properties_destroy (GtkHTMLClassProperties *p)
 	g_free (p);
 }
 
-void
-gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
-{
-}
-
-#define SET(t,x,prop) \
-        { key = g_strconcat (GTK_HTML_GCONF_DIR, x, NULL); \
-        gconf_client_set_ ## t (client, key, p->prop, NULL); \
-        g_free (key); }
-
-void
-gtk_html_class_properties_update (GtkHTMLClassProperties *p, GConfClient *client, GtkHTMLClassProperties *old)
-{
-}
-
 /* enums */
 
 static GEnumValue _gtk_html_cursor_skip_values[] = {
